use crate::zabbix::api::get_zabbix_authtoken;
use crate::zabbix::problems::DataMatrix;
use clap::App;
use nix::sys::inotify::{AddWatchFlags, InitFlags, Inotify};
use serde::{Deserialize, Serialize};
use serde_json::Error as JsonError;
use serde_json::Value as JsonValue;
use std::fs::File;
use std::io::Read;
use std::string::String;
use std::sync::{Arc, Mutex};
use std::time::Duration;
use tokio::time::sleep;

#[derive(Debug)]
pub enum ReloadFrequency {
    High = 50,
    Medium = 20,
    Low = 10,
}

#[derive(Debug, Clone)]
pub struct Context {
    pub cfg: Config,
    pub configfile: String,
    pub inotify: Inotify,
    pub datamatrix: DataMatrix,
    pub mode: String,
}

impl Context {
    pub async fn new() -> Self {
        let yaml = load_yaml!("cli.yml");
        let cli = App::from_yaml(yaml).get_matches();

        let configfile = cli.value_of("config").unwrap_or("config.json").to_string();

        let mut ctx = Context {
            cfg: Config::new(),
            configfile: configfile,
            inotify: Inotify::init(InitFlags::IN_NONBLOCK).unwrap(),
            datamatrix: DataMatrix::new(),
            mode: cli.value_of("mode").unwrap_or("draw").to_string(),
        };

        println!("Loading {configfile} file ...", configfile = ctx.configfile);
        ctx.cfg.load(&ctx.configfile).await;

        println!(
            "Adding inotify watch on {configfile} file ...",
            configfile = ctx.configfile
        );
        ctx.inotify
            .add_watch(ctx.configfile.as_str(), AddWatchFlags::IN_MODIFY)
            .unwrap();
        get_zabbix_authtoken(&mut ctx.cfg).await;

        ctx
    }

    /*pub async fn hotreload(&mut self) {
        let events = match self.inotify.read_events() {
            Ok(ev) => ev,
            Err(_) => vec![],
        };
        if events.len() > 0 {
            println!("Reloading {cfg}", cfg = self.configfile);
            get_zabbix_authtoken(&mut self.cfg).await;
            self.cfg.load(self.configfile.as_str());
        }
        std::thread::sleep(std::time::Duration::from_secs(5));
    }*/
}

pub async fn hotreload(ctx: Context, ctxsender: &tokio::sync::mpsc::Sender<Context>) {
    println!("function hotreload, {:?}", ctxsender);
    let mut sendctx = ctx.clone();
    loop {
        let events = match ctx.inotify.read_events() {
            Ok(ev) => ev,
            Err(_) => vec![],
        };
        if events.len() > 0 {
            sendctx.cfg.load(&sendctx.configfile).await;
        }
        println!("Sending {:?}", ctx);
        ctxsender.send(sendctx.clone()).await;
    }
}

#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct Config {
    pub server: String,
    pub username: String,
    pub password: String,
    #[serde(skip_serializing)]
    pub authtoken: Option<String>,
    pub sloweffect: Option<bool>,
    pub refresh: Option<u64>,
    pub limit: Option<u64>,
}

impl<'a> Config {
    pub fn new() -> Self {
        Self {
            server: "https://zabbix.acme.com/api_jsonrpc.php".to_string(),
            username: "bob".to_string(),
            password: "password".to_string(),
            authtoken: Some("token".to_string()),
            sloweffect: Some(true),
            refresh: Some(5u64),
            limit: Some(20u64),
        }
    }

    async fn load(&mut self, configfile: &str) {
        let mut file = match File::open(configfile) {
            Ok(f) => f,
            Err(err) => {
                panic!("{err}", err = err);
            }
        };
        let mut contents = String::new();
        let fread = file.read_to_string(&mut contents).unwrap();
        println!("{} {} {}", configfile, contents, fread);
        let parse: Result<JsonValue, JsonError> = serde_json::from_str(contents.as_str());
        *self = match parse {
            Ok(cfg) => {
                let tmpcfg = Config::new();
                Config {
                    server: cfg["server"]
                        .as_str()
                        .unwrap_or(tmpcfg.server.as_str())
                        .to_string(),
                    username: cfg["username"]
                        .as_str()
                        .unwrap_or(tmpcfg.username.as_str())
                        .to_string(),
                    password: cfg["password"]
                        .as_str()
                        .unwrap_or(tmpcfg.password.as_str())
                        .to_string(),
                    authtoken: Some(
                        self.authtoken
                            .clone()
                            .unwrap_or(tmpcfg.authtoken.unwrap().to_string())
                            .to_string(),
                    ),
                    sloweffect: Some(
                        cfg["sloweffect"]
                            .as_bool()
                            .unwrap_or(tmpcfg.sloweffect.unwrap()),
                    ),
                    refresh: Some(cfg["refresh"].as_u64().unwrap_or(tmpcfg.refresh.unwrap())),
                    limit: Some(cfg["limit"].as_u64().unwrap_or(tmpcfg.limit.unwrap())),
                }
            }
            Err(err) => {
                println!("error occured: '{}'", err);
                Config::new()
            }
        };
        self.save(&configfile).await;
    }

    async fn save(&self, configfile: &str) {
        let file: File;
        let filemeta = std::fs::metadata(configfile);
        let fileexists = match filemeta {
            Ok(_) => true,
            Err(_) => false,
        };
        if !fileexists {
            file = File::create(configfile).unwrap();
            serde_json::to_writer_pretty(file, &self).unwrap();
            sleep(Duration::from_secs(1)).await;
        }
    }
}
